/**
 * \file TrkLevel2.h
 * \author Elena Vannuccini
 */
#ifndef trklevel2_h
#define trklevel2_h

#include <TObject.h>
#include <TObjArray.h>
#include <TClonesArray.h>
#include <TrkStruct.h>

/**
 * \brief Class to describe, by points, a particle trajectory in the apparatus. 
 *
 * The idea is to create it by integrating the equations of motion, given the 
 * track state vector and the z coordinates where to evaluate track position.
 */
// ==================================================================
class Trajectory : public TObject{
 private:

 public:

    int npoint; ///< number of evaluated points along the trajectory
    float* x;   ///< x coordinates 
    float* y;   ///< y coordinates 
    float* z;   ///< z coordinates 

    Trajectory(){npoint=1; x = new float; y = new float; z = new float; return;};
    Trajectory(int n);
    Trajectory(int n, float* pz);
    void Dump();

    ClassDef(Trajectory,1);

};
/**
 * \brief Class to describe fitted tracks. 
 *
 * A track is defined by the measured coordinates associated to it, the 
 * track status vector, plus other quantities.
 * A track may have an "image", due to the ambiguity in the y view.
 */
// ==================================================================
class TrkTrack : public TObject {

private:

public:

    float al[5];           ///<TRACK STATE VECTOR 
    float coval[5][5];     ///<covariance matrix 
    int   xgood[6];        ///<mask of included x planes 
    int   ygood[6];        ///<mask of included y planes 
    float xm[6];           ///<measured x coordinates
    float ym[6];           ///<measured y coordinates 
    float zm[6];           ///<measured z coordinates 
    float resx[6];         ///<spatial resolution on X view
    float resy[6];         ///<spatial resolution on y view
    float chi2;            ///<chi2
    float xv[6];           ///<calculated x coordinates
    float yv[6];           ///<calculated y coordinates
    float zv[6];           ///<calculated z coordinates
    float axv[6];          ///<calculated angles (deg) on x view
    float ayv[6];          ///<calculated angles (deg) on y view
    float dedx_x[6];       ///<signal in MIP (scaled to 300 micrometer)
    float dedx_y[6];       ///<signal in MIP (scaled to 300 micrometer)
    int   image;           ///<flag to tag track-images 

    TrkTrack();
    TrkTrack(const TrkTrack&);

    void Dump();

    Bool_t HasImage(){return !(image==-1);} ///< Returns true if the track has an image
	int DoTrack(Trajectory* t);   			///< Evaluates the trajectory in the apparatus.
	float BdL(){return 0;};					///< Evaluates the integral of B*dL along the track.
    Int_t GetNX(){Int_t n=0; for(Int_t i=0; i<6; i++)n+=xgood[i]; return n;}; 
    Int_t GetNY(){Int_t n=0; for(Int_t i=0; i<6; i++)n+=ygood[i]; return n;};
    Float_t GetRigidity();
    Float_t GetDeflection();
    Float_t GetDEDX();

    TrkTrack* GetTrkTrack(){return this;};

    ClassDef(TrkTrack,1);

};
/**
 * \brief Class to describe single clusters ("singlets"). 
 *
 * Single clusters are clusters not associated to any track.
 */
class TrkSinglet : public TObject {

private:

public:

    int plane;       ///<plane 
    float coord[2];  ///<coordinate (on sensor 1 and 2)
    float sgnl;      ///<cluster signal in MIP 

    TrkSinglet();
    TrkSinglet(const TrkSinglet&);

    void Dump();

    ClassDef(TrkSinglet,1);

};

/**
 * \brief Class to describe tracker LEVEL2 data.
 *
 * A tracker events is defined by some general variables, plus the collection of all the fitted tracks and all 
 * single clusters on X and Y views. 
 * Tracks and single clusters ("singlets") are described by the classes TrkTrack and TrkSinglet respectivelly.
 * 
 * Each track may have an "image", due to the ambiguity on the Y view, which is stored also. 
 * Thus, the number of stored tracks ( ntrk() ) differs from the number of "physical" tracks ( GetNTracks() ). 
 * Proper methods allow to sort tracks and select the physical ones ( GetTracks() ).
 */
class TrkLevel2 : public TObject {

 private:

 public:



    Int_t good2;
    Int_t crc[12];

    TClonesArray *Track;        ///< fitted tracks
    TClonesArray *SingletX;     ///< x singlets
    TClonesArray *SingletY;     ///< y singlets

    TrkLevel2();
//    TrkLevel2(cTrkLevel2 *);

    int ntrk(){return Track->GetEntries();}     ///< number of stored track
    int nclsx(){return SingletX->GetEntries();} ///< number of x singlets 
    int nclsy(){return SingletY->GetEntries();} ///< number of y singlets 

    void Dump();
    void FillCommonVar(cTrkLevel2 *);
    void Clear();
	void LoadField(TString);

    TrkTrack *GetStoredTrack(int i);
    TClonesArray *GetTracks();  

    int GetNTracks(){return this->GetTracks()->GetEntries();}
    TrkTrack* GetTrack(int i);
    TrkTrack* GetTrackImage(int i);

    TrkLevel2* GetTrkLevel2(){return this;}

    ClassDef(TrkLevel2,1);

};




#endif
